@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _BRL_DRIVER_H
-#define _BRL_DRIVER_H
+#ifndef BRLTTY_INCLUDED_BRL_DRIVER
+#define BRLTTY_INCLUDED_BRL_DRIVER
 
 #ifdef __cplusplus
 extern "C" {
@@ -24,16 +24,14 @@ extern "C" {
 
 /* this header file is used to create the driver structure
  * for a dynamically loadable braille display driver.
- * BRLNAME, BRLCODE, BRLHELP, and BRLSTAT must be defined - see driver make file
  */
 
 #include "brl.h"
 
 /* Routines provided by this braille display driver. */
-static void brl_identify (void);
-static int brl_open (BrailleDisplay *, char **parameters, const char *);
-static void brl_close (BrailleDisplay *);
-static int brl_readCommand (BrailleDisplay *, DriverCommandContext);
+static int brl_construct (BrailleDisplay *, char **parameters, const char *);
+static void brl_destruct (BrailleDisplay *);
+static int brl_readCommand (BrailleDisplay *, BRL_DriverCommandContext);
 static void brl_writeWindow (BrailleDisplay *);
 static void brl_writeStatus (BrailleDisplay *brl, const unsigned char *);
 
@@ -42,16 +40,24 @@ static void brl_writeStatus (BrailleDisplay *brl, const unsigned char *);
 #endif /* BRL_HAVE_VISUAL_DISPLAY */
 
 #ifdef BRL_HAVE_PACKET_IO
-  static int brl_readPacket (BrailleDisplay *, unsigned char *, int);
-  static int brl_writePacket (BrailleDisplay *, const unsigned char *, int);
+  static ssize_t brl_readPacket (BrailleDisplay *, void *, size_t);
+  static ssize_t brl_writePacket (BrailleDisplay *, const void *, size_t);
   static int brl_reset (BrailleDisplay *);
 #endif /* BRL_HAVE_PACKET_IO */
 
 #ifdef BRL_HAVE_KEY_CODES
   static int brl_readKey (BrailleDisplay *);
-  static int brl_keyToCommand (BrailleDisplay *, DriverCommandContext, int);
+  static int brl_keyToCommand (BrailleDisplay *, BRL_DriverCommandContext, int);
 #endif /* BRL_HAVE_KEY_CODES */
 
+#ifdef BRL_HAVE_FIRMNESS
+  static void brl_firmness (BrailleDisplay *brl, BrailleFirmness setting);
+#endif /* BRL_HAVE_FIRMNESS */
+
+#ifdef BRL_HAVE_SENSITIVITY
+  static void brl_sensitivity (BrailleDisplay *brl, BrailleSensitivity setting);
+#endif /* BRL_HAVE_SENSITIVITY */
+
 #ifdef BRLPARMS
   static const char *const brl_parameters[] = {BRLPARMS, NULL};
 #endif /* BRLPARMS */
@@ -61,7 +67,7 @@ static void brl_writeStatus (BrailleDisplay *brl, const unsigned char *);
 #endif /* BRLSTAT */
 
 #ifndef BRLSYMBOL
-#  define BRLSYMBOL CONCATENATE(brl_driver_,BRLCODE)
+#  define BRLSYMBOL CONCATENATE(brl_driver_,DRIVER_CODE)
 #endif /* BRLSYMBOL */
 
 #ifndef BRLCONST
@@ -70,8 +76,7 @@ static void brl_writeStatus (BrailleDisplay *brl, const unsigned char *);
 
 extern BRLCONST BrailleDriver BRLSYMBOL;
 BRLCONST BrailleDriver BRLSYMBOL = {
-  STRINGIFY(BRLNAME),
-  STRINGIFY(BRLCODE),
+  DRIVER_DEFINITION_INITIALIZER,
 
 #ifdef BRLPARMS
   brl_parameters,
@@ -82,9 +87,8 @@ BRLCONST BrailleDriver BRLSYMBOL = {
   BRLHELP,
   BRLSTAT,
 
-  brl_identify,
-  brl_open,
-  brl_close,
+  brl_construct,
+  brl_destruct,
   brl_readCommand,
   brl_writeWindow,
   brl_writeStatus,
@@ -107,15 +111,29 @@ BRLCONST BrailleDriver BRLSYMBOL = {
 
 #ifdef BRL_HAVE_KEY_CODES
   brl_readKey,
-  brl_keyToCommand
+  brl_keyToCommand,
 #else /* BRL_HAVE_KEY_CODES */
   NULL, /* brl_readKey */
-  NULL  /* brl_keyToCommand */
+  NULL, /* brl_keyToCommand */
 #endif /* BRL_HAVE_KEY_CODES */
+
+#ifdef BRL_HAVE_FIRMNESS
+  brl_firmness,
+#else /* BRL_HAVE_FIRMNESS */
+  NULL, /* brl_firmness */
+#endif /* BRL_HAVE_FIRMNESS */
+
+#ifdef BRL_HAVE_SENSITIVITY
+  brl_sensitivity
+#else /* BRL_HAVE_SENSITIVITY */
+  NULL  /* brl_sensitivity */
+#endif /* BRL_HAVE_SENSITIVITY */
 };
 
+DRIVER_VERSION_DECLARATION(brl);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* _BRL_DRIVER_H */
+#endif /* BRLTTY_INCLUDED_BRL_DRIVER */
