@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,24 +15,20 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <locale.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
  
 #include "misc.h"
-#include "contract.h"
-#include "ctb_definitions.h"
+#include "ctb.h"
+#include "ctb_internal.h"
 #include "brl.h"
 
-
 typedef struct {
   BYTE length;
   BYTE bytes[0XFF];
@@ -44,9 +40,70 @@ static ContractionTableHeader *tableHeader;
 static ContractionTableOffset tableSize;
 static ContractionTableOffset tableUsed;
 
-static BYTE *opcodesTable;
-static int opcodesSize;
-static int opcodesUsed;
+static const char *const characterClassNames[] = {
+  "space",
+  "letter",
+  "digit",
+  "punctuation",
+  "uppercase",
+  "lowercase",
+  NULL
+};
+struct CharacterClass {
+  struct CharacterClass *next;
+  ContractionTableCharacterAttributes attribute;
+  BYTE length;
+  char name[1];
+};
+static struct CharacterClass *characterClasses;
+static ContractionTableCharacterAttributes characterClassAttribute;
+
+static const char *const opcodeNames[CTO_None] = {
+  "include",
+  "locale",
+
+  "capsign",
+  "begcaps",
+  "endcaps",
+
+  "letsign",
+  "numsign",
+
+  "literal",
+  "replace",
+  "always",
+  "repeated",
+
+  "largesign",
+  "lastlargesign",
+  "word",
+  "joinword",
+  "lowword",
+  "contraction",
+
+  "sufword",
+  "prfword",
+  "begword",
+  "begmidword",
+  "midword",
+  "midendword",
+  "endword",
+
+  "prepunc",
+  "postpunc",
+
+  "begnum",
+  "midnum",
+  "endnum",
+
+  "class",
+  "after",
+  "before"
+};
+static unsigned char opcodeLengths[CTO_None] = {0};
+
+static char *originalLocale;
+static int noLocale;
 
 typedef struct {
   const char *fileName;
@@ -55,17 +112,26 @@ typedef struct {
 
 static void
 compileError (FileData *data, char *format, ...) {
-  char buffer[0X100];
-  va_list arguments;
-  va_start(arguments, format);
-  vsnprintf(buffer, sizeof(buffer), format, arguments);
-  va_end(arguments);
+  char message[0X100];
 
-  if (data)
-    LogPrint(LOG_WARNING, "File %s line %d: %s",
-             data->fileName, data->lineNumber, buffer);
-  else
-    LogPrint(LOG_WARNING, "%s", buffer);
+  {
+    const char *file = NULL;
+    const int *line = NULL;
+    va_list args;
+
+    if (data) {
+      file = data->fileName;
+      if (data->lineNumber) line = &data->lineNumber;
+    }
+
+    va_start(args, format);
+    formatInputError(message, sizeof(message),
+                     file, line,
+                     format, args);
+    va_end(args);
+  }
+
+  LogPrint(LOG_WARNING, "%s", message);
   errorCount++;
 }
 
@@ -88,137 +154,159 @@ allocateBytes (FileData *data, ContractionTableOffset *offset, int count, int al
 }
 
 static int
-addEntry (FileData *data, ContractionTableOpcode opcode, ByteString *find, ByteString *replace) {
-  ContractionTableOffset entryOffset;
-  int entrySize = sizeof(ContractionTableEntry) - 1;
-  if (find) entrySize += find->length;
-  if (replace) entrySize += replace->length;
-  if (allocateBytes(data, &entryOffset, entrySize, __alignof__(ContractionTableEntry))) {
-    ContractionTableEntry *newEntry = CTE(tableHeader, entryOffset);
-    newEntry->opcode = opcode;
-    if (find)
-      memcpy(&newEntry->findrep[0], &find->bytes[0],
-             (newEntry->findlen = find->length));
-    else
-      newEntry->findlen = 0;
-    if (replace)
-      memcpy(&newEntry->findrep[newEntry->findlen], &replace->bytes[0],
-             (newEntry->replen = replace->length));
-    else
-      newEntry->replen = 0;
-    newEntry->next = 0;
-
-    /*link new entry into table.*/
-    if (newEntry->findlen) {
-      ContractionTableOffset bucket;
-
-      /* first, handle single-character entries. */
-      if (newEntry->findlen == 1) { /*it's a cup, not a bucket */
-        BYTE character = newEntry->findrep[0];
-        if (newEntry->opcode == CTO_Always) {
-          tableHeader->characters[character].entry = entryOffset;
-        }
-        if ((bucket = tableHeader->cups[character])) {
-          ContractionTableEntry *currentEntry = CTE(tableHeader, bucket);
-          while (currentEntry->next)
-            currentEntry = CTE(tableHeader, currentEntry->next);
-          currentEntry->next = entryOffset;
-        } else {
-          tableHeader->cups[character] = entryOffset;
-        }
-      } else {
-        /* Now, work through the various cases for multi-byte find strings. */
-        bucket = hash(newEntry->findrep);
-
-        /* case 1, start new hash chain. */
-        if (tableHeader->buckets[bucket] == 0) {
-          tableHeader->buckets[bucket] = entryOffset;
-        } else {
-          /* Case 2, longest entry goes at head of chain. */
-          ContractionTableEntry *currentEntry = CTE(tableHeader, tableHeader->buckets[bucket]);
-          if (newEntry->findlen > currentEntry->findlen) {
-            newEntry->next = tableHeader->buckets[bucket];
-            tableHeader->buckets[bucket] = entryOffset;
-          } else {
-            /* Case 3, new entry goes somewhere in chain. */
-            while (currentEntry->next) { /*loop through chain */
-              ContractionTableEntry *previousEntry = currentEntry;	/*also used in sorting */
-              currentEntry = CTE(tableHeader, currentEntry->next);
-              if (newEntry->findlen > currentEntry->findlen) {
-                newEntry->next = previousEntry->next;
-                previousEntry->next = entryOffset;
-                break;
-              }
-            }
-            if (!newEntry->next) {
-              /* Case 4, new entry goes at end of chain*/
-              currentEntry->next = entryOffset;
-            }
-          }
-        }
-      }
-    }
-
+saveBytes (FileData *data, ContractionTableOffset *offset, const BYTE *bytes, BYTE length) {
+  if (allocateBytes(data, offset, length+1, __alignof__(BYTE))) {
+    BYTE *address = CTA(tableHeader, *offset);
+    memcpy(address+1, bytes, (*address = length));
     return 1;
   }
   return 0;
 }
 
 static int
-addString (FileData *data, ContractionTableOffset *offset, ByteString *string) {
-  if (allocateBytes(data, offset, string->length+1, __alignof__(BYTE))) {
-    BYTE *address = CTA(tableHeader, *offset);
-    memcpy(address+1, string->bytes, (*address = string->length));
-    return 1;
+saveString (FileData *data, ContractionTableOffset *offset, const char *string) {
+  return saveBytes(data, offset, (const unsigned char *)string, strlen(string));
+}
+
+static int
+saveByteString (FileData *data, ContractionTableOffset *offset, const ByteString *string) {
+  return saveBytes(data, offset, string->bytes, string->length);
+}
+
+static ContractionTableRule *
+addRule (
+  FileData *data,
+  ContractionTableOpcode opcode,
+  ByteString *find,
+  ByteString *replace,
+  ContractionTableCharacterAttributes after,
+  ContractionTableCharacterAttributes before
+) {
+  ContractionTableOffset ruleOffset;
+  int ruleSize = sizeof(ContractionTableRule) - 1;
+  if (find) ruleSize += find->length;
+  if (replace) ruleSize += replace->length;
+  if (allocateBytes(data, &ruleOffset, ruleSize, __alignof__(ContractionTableRule))) {
+    ContractionTableRule *newRule = CTR(tableHeader, ruleOffset);
+
+    newRule->opcode = opcode;
+    newRule->after = after;
+    newRule->before = before;
+    if (find)
+      memcpy(&newRule->findrep[0], &find->bytes[0],
+             (newRule->findlen = find->length));
+    else
+      newRule->findlen = 0;
+    if (replace)
+      memcpy(&newRule->findrep[newRule->findlen], &replace->bytes[0],
+             (newRule->replen = replace->length));
+    else
+      newRule->replen = 0;
+
+    /*link new rule into table.*/
+    {
+      ContractionTableOffset *offsetAddress;
+
+      if (newRule->findlen == 1) {
+        ContractionTableCharacter *character = &tableHeader->characters[newRule->findrep[0]];
+        if (newRule->opcode == CTO_Always) character->always = ruleOffset;
+        offsetAddress = &character->rules;
+      } else {
+        offsetAddress = &tableHeader->rules[hash(newRule->findrep)];
+      }
+
+      while (*offsetAddress) {
+        ContractionTableRule *currentRule = CTR(tableHeader, *offsetAddress);
+        if (newRule->findlen > currentRule->findlen) break;
+        if (newRule->findlen == currentRule->findlen) {
+          if ((currentRule->opcode == CTO_Always) && (newRule->opcode != CTO_Always)) break;
+        }
+        offsetAddress = &currentRule->next;
+      }
+
+      newRule->next = *offsetAddress;
+      *offsetAddress = ruleOffset;
+    }
+
+    return newRule;
   }
-  return 0;
+  return NULL;
+}
+
+static const struct CharacterClass *
+findCharacterClass (const unsigned char *name, int length) {
+  const struct CharacterClass *class = characterClasses;
+  while (class) {
+    if ((length == class->length) &&
+        (memcmp(name, class->name, length) == 0))
+      return class;
+    class = class->next;
+  }
+  return NULL;
+}
+
+static struct CharacterClass *
+addCharacterClass (FileData *data, const unsigned char *name, int length) {
+  struct CharacterClass *class;
+  if (characterClassAttribute) {
+    if ((class = malloc(sizeof(*class) + length - 1))) {
+      memset(class, 0, sizeof(*class));
+      memcpy(class->name, name, (class->length = length));
+
+      class->attribute = characterClassAttribute;
+      characterClassAttribute <<= 1;
+
+      class->next = characterClasses;
+      characterClasses = class;
+      return class;
+    }
+  }
+  compileError(data, "character class table overflow: %.*s", length, name);
+  return NULL;
+}
+
+static void
+deallocateCharacterClasses (void) {
+  while (characterClasses) {
+    struct CharacterClass *class = characterClasses;
+    characterClasses = characterClasses->next;
+    free(class);
+  }
+}
+
+static int
+allocateCharacterClasses (void) {
+  const char *const *name = characterClassNames;
+  characterClasses = NULL;
+  characterClassAttribute = 1;
+  while (*name) {
+    if (!addCharacterClass(NULL, (const unsigned char *)*name, strlen(*name))) {
+      deallocateCharacterClasses();
+      return 0;
+    }
+    ++name;
+  }
+  return 1;
 }
 
 static ContractionTableOpcode
-getOpcode (FileData *data, const char *token, int length) {
-  BYTE *entry = opcodesTable;		/*pointer for looking things up in opcodesTable */
-  if (entry) {
-    const BYTE *end = entry + opcodesUsed;
-    while (entry < end) {
-      BYTE len = *entry++;
-      if (len == length)
-        if (memcmp(entry, token, length) == 0)
-          return *(entry + len);
-      entry += len + 1;
-    }
-  }
+getOpcode (FileData *data, const unsigned char *token, int length) {
+  ContractionTableOpcode opcode;
+  for (opcode=0; opcode<CTO_None; ++opcode)
+    if (length == opcodeLengths[opcode])
+      if (memcmp(token, opcodeNames[opcode], length) == 0)
+        return opcode;
   compileError(data, "opcode not defined: %.*s", length, token);
   return CTO_None;
 }
 
 static int
-addOpcode (FileData *data, const char *token, int length, ContractionTableOpcode opcode) {
-  int size = opcodesUsed + length + 2;
-  if (size > opcodesSize) {
-    void *table = realloc(opcodesTable, size|=0XFF);
-    if (!table) {
-      compileError(data, "Not enough memory for opcodes table.");
-      return 0;
-    }
-    opcodesTable = table;
-    opcodesSize = size;
-  }
-  opcodesTable[opcodesUsed++] = length;
-  memcpy(&opcodesTable[opcodesUsed], token, length);
-  opcodesUsed += length;
-  opcodesTable[opcodesUsed++] = opcode;
-  return 1;
-}
-
-static int
-getToken (FileData *data, const BYTE **token, int *length, const char *description) { /*find the next string of contiguous nonblank characters */
-  const BYTE *start = *token + *length;
+getToken (FileData *data, const unsigned char **token, int *length, const char *description) { /*find the next string of contiguous nonblank characters */
+  const unsigned char *start = *token + *length;
   int count = 0;
 
-  while (isspace(*start))
-    start++;
-  while (start[count] && !isspace(start[count]))
-    count++;
+  while (isspace(*start)) start++;
+  while (start[count] && !isspace(start[count])) count++;
   *token = start;
   if (!(*length = count)) {
     if (description)
@@ -229,7 +317,7 @@ getToken (FileData *data, const BYTE **token, int *length, const char *descripti
 }				/*token obtained */
 
 static int
-hexadecimalDigit (FileData *data, int *value, BYTE digit) {
+hexadecimalDigit (FileData *data, int *value, char digit) {
   if (digit >= '0' && digit <= '9')
     *value = digit - '0';
   else if (digit >= 'a' && digit <= 'f')
@@ -242,32 +330,32 @@ hexadecimalDigit (FileData *data, int *value, BYTE digit) {
 }
 
 static int
-octalDigit (FileData *data, int *value, BYTE digit) {
+octalDigit (FileData *data, int *value, char digit) {
   if (digit < '0' || digit > '7') return 0;
   *value = digit - '0';
   return 1;
 }
 
 static int
-parseText (FileData *data, ByteString *result, const BYTE *token, const int length) {	/*interpret find string */
+parseText (FileData *data, ByteString *result, const unsigned char *token, const int length) {	/*interpret find string */
   int count = 0;		/*loop counters */
   int index;		/*loop counters */
   for (index = 0; index < length; index++) {
-    BYTE byte = token[index];
-    if (byte == '\\') { /* escape sequence */
+    char character = token[index];
+    if (character == '\\') { /* escape sequence */
       int ok = 0;
       int start = index;
       if (++index < length) {
-        switch (byte = token[index]) {
+        switch (character = token[index]) {
           case '\\':
             ok = 1;
             break;
           case 'f':
-            byte = '\f';
+            character = '\f';
             ok = 1;
             break;
           case 'n':
-            byte = '\n';
+            character = '\n';
             ok = 1;
             break;
           case 'o':
@@ -277,25 +365,25 @@ parseText (FileData *data, ByteString *result, const BYTE *token, const int leng
                 if (high < 04)
                   if (octalDigit(data, &middle, token[++index]))
                     if (octalDigit(data, &low, token[++index])) {
-                      byte = (high << 6) | (middle << 3) | low;
+                      character = (high << 6) | (middle << 3) | low;
                       ok = 1;
                     }
             }
             break;
           case 'r':
-            byte = '\r';
+            character = '\r';
             ok = 1;
             break;
           case 's':
-            byte = ' ';
+            character = ' ';
             ok = 1;
             break;
           case 't':
-            byte = '\t';
+            character = '\t';
             ok = 1;
             break;
           case 'v':
-            byte = '\v';
+            character = '\v';
             ok = 1;
             break;
           case 'x':
@@ -303,7 +391,7 @@ parseText (FileData *data, ByteString *result, const BYTE *token, const int leng
               int high, low;
               if (hexadecimalDigit(data, &high, token[++index]))
                 if (hexadecimalDigit(data, &low, token[++index])) {
-                  byte = (high << 4) | low;
+                  character = (high << 4) | low;
                   ok = 1;
                 }
             }
@@ -317,17 +405,15 @@ parseText (FileData *data, ByteString *result, const BYTE *token, const int leng
         return 0;
       }
     }
-    if (!byte) {
-      byte = ' ';
-    }
-    result->bytes[count++] = byte;
+    if (!character) character = ' ';
+    result->bytes[count++] = character;
   }
   result->length = count;
   return 1;
 }				/*find string interpreted */
 
 static int
-parseDots (FileData *data, ByteString *cells, const BYTE *token, const int length) {	/*get dot patterns */
+parseDots (FileData *data, ByteString *cells, const unsigned char *token, const int length) {	/*get dot patterns */
   BYTE cell = 0;		/*assembly place for dots */
   int count = 0;		/*loop counters */
   int index;		/*loop counters */
@@ -335,37 +421,37 @@ parseDots (FileData *data, ByteString *cells, const BYTE *token, const int lengt
 
   for (index = 0; index < length; index++) {
     int started = index != start;
-    BYTE byte = token[index];
-    switch (byte) { /*or dots to make up Braille cell */
+    char character = token[index];
+    switch (character) { /*or dots to make up Braille cell */
       {
         int dot;
       case '1':
-        dot = B1;
+        dot = BRL_DOT1;
         goto haveDot;
       case '2':
-        dot = B2;
+        dot = BRL_DOT2;
         goto haveDot;
       case '3':
-        dot = B3;
+        dot = BRL_DOT3;
         goto haveDot;
       case '4':
-        dot = B4;
+        dot = BRL_DOT4;
         goto haveDot;
       case '5':
-        dot = B5;
+        dot = BRL_DOT5;
         goto haveDot;
       case '6':
-        dot = B6;
+        dot = BRL_DOT6;
         goto haveDot;
       case '7':
-        dot = B7;
+        dot = BRL_DOT7;
         goto haveDot;
       case '8':
-        dot = B8;
+        dot = BRL_DOT8;
       haveDot:
         if (started && !cell) goto invalid;
         if (cell & dot) {
-          compileError(data, "dot specified more than once: %c", byte);
+          compileError(data, "dot specified more than once: %c", character);
           return 0;
         }
         cell |= dot;
@@ -386,7 +472,7 @@ parseDots (FileData *data, ByteString *cells, const BYTE *token, const int lengt
         break;
       default:
       invalid:
-        compileError(data, "invalid dot number: %c", byte);
+        compileError(data, "invalid dot number: %c", character);
         return 0;
     }
   }
@@ -400,7 +486,15 @@ parseDots (FileData *data, ByteString *cells, const BYTE *token, const int lengt
 }				/*end of function parseDots */
 
 static int
-getFindText (FileData *data, ByteString *find, const BYTE **token, int *length) {
+getCharacters (FileData *data, ByteString *characters, const unsigned char **token, int *length) {
+  if (getToken(data, token, length, "characters"))
+    if (parseText(data, characters, *token, *length))
+      return 1;
+  return 0;
+}
+
+static int
+getFindText (FileData *data, ByteString *find, const unsigned char **token, int *length) {
   if (getToken(data, token, length, "find text"))
     if (parseText(data, find, *token, *length))
       return 1;
@@ -408,7 +502,7 @@ getFindText (FileData *data, ByteString *find, const BYTE **token, int *length)
 }
 
 static int
-getReplaceText (FileData *data, ByteString *replace, const BYTE **token, int *length) {
+getReplaceText (FileData *data, ByteString *replace, const unsigned char **token, int *length) {
   if (getToken(data, token, length, "replacement text"))
     if (parseText(data, replace, *token, *length))
       return 1;
@@ -416,7 +510,7 @@ getReplaceText (FileData *data, ByteString *replace, const BYTE **token, int *le
 }
 
 static int
-getReplacePattern (FileData *data, ByteString *replace, const BYTE **token, int *length) {
+getReplacePattern (FileData *data, ByteString *replace, const unsigned char **token, int *length) {
   if (getToken(data, token, length, "replacement pattern")) {
     if (*length == 1 && **token == '=') {
       replace->length = 0;
@@ -428,51 +522,38 @@ getReplacePattern (FileData *data, ByteString *replace, const BYTE **token, int
   return 0;
 }
 
-static int
-integerToken (FileData *data, int *integer, const char *token, int length,
-              const char *description, const int *minimum, const int *maximum) {
-  char *end;
-  unsigned long value = strtol(token, &end, 0);
-  if (!minimum) {
-    static const int limit = 0;
-    minimum = &limit;
-  }
-  if (!maximum) {
-    static const int limit = 0X7FFFFFFF;
-    maximum = &limit;
-  }
-  if ((end != (token + length)) || (value < *minimum) || (value > *maximum)) {
-    compileError(data, "invalid %s: %.*s", description, length, token);
-    return 0;
-  }
-  *integer = value;
-  return 1;
+static const char *
+setLocale (const char *locale) {
+  return setlocale(LC_CTYPE, locale);
 }
 
 static int
-opcodeToken (FileData *data, ContractionTableOpcode *opcode, const char *token, int length) {
-  static const int minimum = 0;
-  static const int maximum = CTO_None - 1;
-  int integer;
-  if (!integerToken(data, &integer, token, length, "opcode number", &minimum, &maximum))
-    return 0;
-  *opcode = integer;
-  return 1;
+getCharacterClass (FileData *data, const struct CharacterClass **class, const unsigned char **token, int *length) {
+  if (getToken(data, token, length, "character class name")) {
+    if ((*class = findCharacterClass(*token, *length))) return 1;
+    compileError(data, "character class not defined: %.*s", *length, *token);
+  }
+  return 0;
+}
+
+static const char *
+getLocale (void) {
+  return setLocale(NULL);
 }
 
 static int processFile (const char *fileName);
 static int
 includeFile (FileData *data, ByteString *path) {
-  const BYTE *prefixAddress = data->fileName;
+  const char *prefixAddress = data->fileName;
   int prefixLength = 0;
-  const BYTE *suffixAddress = path->bytes;
+  const unsigned char *suffixAddress = path->bytes;
   int suffixLength = path->length;
   if (*suffixAddress != '/') {
-    const BYTE *ptr = strrchr(prefixAddress, '/');
+    const char *ptr = strrchr(prefixAddress, '/');
     if (ptr) prefixLength = ptr - prefixAddress + 1;
   }
   {
-    BYTE file[prefixLength + suffixLength + 1];
+    char file[prefixLength + suffixLength + 1];
     snprintf(file, sizeof(file), "%.*s%.*s",
              prefixLength, prefixAddress, suffixLength, suffixAddress);
     return processFile(file);
@@ -480,22 +561,23 @@ includeFile (FileData *data, ByteString *path) {
 }
 
 static int
-processLine (FileData *data, const BYTE *line) {
+processLine (FileData *data, const char *line) {
   int ok = 0;
-  const BYTE *token = line;
+  const unsigned char *token = (const unsigned char *)line;
   int length = 0;			/*length of token */
   ContractionTableOpcode opcode;
+  ContractionTableCharacterAttributes after = 0;
+  ContractionTableCharacterAttributes before = 0;
 
+doOpcode:
   if (!getToken(data, &token, &length, NULL)) return 1;			/*blank line */
   if (*token == '#') return 1;
+  opcode = getOpcode(data, token, length);
 
-  if (*token < '0' || *token > '9') { /*look up word in opcode table */
-    opcode = getOpcode(data, token, length);
-  } else if (!opcodeToken(data, &opcode, token, length))
-    return 1;
   switch (opcode) { /*Carry out operations */
     case CTO_None:
       break;
+
     case CTO_IncludeFile: {
       ByteString path;
       if (getToken(data, &token, &length, "include file path"))
@@ -504,97 +586,155 @@ processLine (FileData *data, const BYTE *line) {
             goto failure;
       break;
     }
-    case CTO_Synonym:
-      if (getToken(data, &token, &length, "opcode number"))
-        if (opcodeToken(data, &opcode, token, length))
-          if (getToken(data, &token, &length, "opcode name"))
-            if (opcode != CTO_None)
-              if (!addOpcode(data, token, length, opcode))
-                goto failure;
+
+    case CTO_Locale: {
+      ByteString locale;
+      if (getToken(data, &token, &length, "locale specification"))
+        if (parseText(data, &locale, token, length)) {
+          char string[locale.length + 1];
+          snprintf(string, sizeof(string), "%.*s", locale.length, locale.bytes);
+          if (strcmp(string, "-") == 0) {
+            noLocale = 1;
+          } else if (!setLocale(string)) {
+            compileError(data, "locale not available: %s", string);
+          } else {
+            noLocale = 0;
+          }
+        }
       break;
+    }
+
     case CTO_Always:
-    case CTO_WholeWord:
-    case CTO_LowWord:
-    case CTO_JoinableWord:
     case CTO_LargeSign:
+    case CTO_LastLargeSign:
+    case CTO_WholeWord:
+    case CTO_JoinableWord:
+    case CTO_LowWord:
     case CTO_SuffixableWord:
+    case CTO_PrefixableWord:
     case CTO_BegWord:
+    case CTO_BegMidWord:
     case CTO_MidWord:
     case CTO_MidEndWord:
     case CTO_EndWord:
+    case CTO_PrePunc:
+    case CTO_PostPunc:
     case CTO_BegNum:
     case CTO_MidNum:
     case CTO_EndNum:
-    case CTO_PrePunc:
-    case CTO_PostPunc:
     case CTO_Repeated: {
       ByteString find;
       ByteString replace;
       if (getFindText(data, &find, &token, &length))
         if (getReplacePattern(data, &replace, &token, &length))
-          if (!addEntry(data, opcode, &find, &replace))
+          if (!addRule(data, opcode, &find, &replace, after, before))
             goto failure;
       break;
     }
+
     case CTO_Replace: {
       ByteString find;
       ByteString replace;
       if (getFindText(data, &find, &token, &length))
         if (getReplaceText(data, &replace, &token, &length))
-          if (!addEntry(data, opcode, &find, &replace))
+          if (!addRule(data, opcode, &find, &replace, after, before))
             goto failure;
       break;
     }
+
     case CTO_Contraction:
     case CTO_Literal: {
       ByteString find;
       if (getFindText(data, &find, &token, &length))
-        if (!addEntry(data, opcode, &find, NULL))
+        if (!addRule(data, opcode, &find, NULL, after, before))
           goto failure;
       break;
     }
+
     case CTO_CapitalSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "capital sign"))
         if (parseDots(data, &cells, token, length))
-          if (!addString(data, &tableHeader->capitalSign, &cells))
+          if (!saveByteString(data, &tableHeader->capitalSign, &cells))
             goto failure;
       break;
     }
+
     case CTO_BeginCapitalSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "begin capital sign"))
         if (parseDots(data, &cells, token, length))
-          if (!addString(data, &tableHeader->beginCapitalSign, &cells))
+          if (!saveByteString(data, &tableHeader->beginCapitalSign, &cells))
             goto failure;
       break;
     }
+
     case CTO_EndCapitalSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "end capital sign"))
         if (parseDots(data, &cells, token, length))
-          if (!addString(data, &tableHeader->endCapitalSign, &cells))
+          if (!saveByteString(data, &tableHeader->endCapitalSign, &cells))
             goto failure;
       break;
     }
+
     case CTO_EnglishLetterSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "letter sign"))
         if (parseDots(data, &cells, token, length))
-          if (!addString(data, &tableHeader->englishLetterSign, &cells))
+          if (!saveByteString(data, &tableHeader->englishLetterSign, &cells))
             goto failure;
       break;
     }
+
     case CTO_NumberSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "number sign"))
         if (parseDots(data, &cells, token, length))
-          if (!addString(data, &tableHeader->numberSign, &cells))
+          if (!saveByteString(data, &tableHeader->numberSign, &cells))
             goto failure;
       break;
     }
+
+    case CTO_Class: {
+      const struct CharacterClass *class;
+      ByteString characters;
+      if (getToken(data, &token, &length, "character class name")) {
+        if ((class = findCharacterClass(token, length))) {
+          compileError(data, "character class already defined: %.*s", length, token);
+        } else if ((class = addCharacterClass(data, token, length))) {
+          if (getCharacters(data, &characters, &token, &length)) {
+            int index;
+            for (index=0; index<characters.length; ++index) {
+              ContractionTableCharacter *character = &tableHeader->characters[characters.bytes[index]];
+              character->attributes |= class->attribute;
+            }
+          }
+        }
+      }
+      break;
+    }
+
+    {
+      ContractionTableCharacterAttributes *attributes;
+      const struct CharacterClass *class;
+
+    case CTO_After:
+      attributes = &after;
+      goto doClass;
+    case CTO_Before:
+      attributes = &before;
+    doClass:
+
+      if (getCharacterClass(data, &class, &token, &length)) {
+        *attributes |= class->attribute;
+        goto doOpcode;
+      }
+      break;
+    }
+
     default:
-      compileError(data, "unimplemented opcode: %d", opcode);
+      compileError(data, "unimplemented opcode: %.*s", length, token);
       break;
   }				/*end of loop for processing tableStream */
   ok = 1;
@@ -613,12 +753,12 @@ processFile (const char *fileName) {
   data.lineNumber = 0;
   LogPrint(LOG_DEBUG, "Including contraction table: %s", fileName);
 
-  if ((stream = fopen(data.fileName, "r"))) { /*fatal error */
+  if ((stream = openDataFile(data.fileName, "r"))) {
     int size = 0X80;
-    BYTE *buffer = malloc(size);
+    char *buffer = malloc(size);
     if (buffer) {
       while (1) { /*Process lines in table */
-        BYTE *line = fgets(buffer, size, stream);
+        char *line = fgets(buffer, size, stream);
         int length;
         if (!line) {
           if (ferror(stream))
@@ -629,7 +769,7 @@ processFile (const char *fileName) {
 
         while (buffer[(length = strlen(line)) - 1] != '\n') {
           if (length + 1 == size) {
-            BYTE *newBuffer = realloc(buffer, size<<=1);
+            char *newBuffer = realloc(buffer, size<<=1);
             if (!newBuffer) {
               compileError(&data, "cannot extend input buffer.");
               ok = 0;
@@ -700,7 +840,7 @@ auditCharacters (BYTE *encountered, const BYTE *start, BYTE length) {
    BYTE count = length;
   while (count--) {
     BYTE character = *address++;
-    if (!tableHeader->characters[character].entry && !encountered[character]) {
+    if (!tableHeader->characters[character].always && !encountered[character]) {
       encountered[character] = 1;
       compileError(NULL, "character representation not defined: %c (%.*s)",
                    character, length, start);
@@ -717,26 +857,26 @@ auditTable (void) {
   int hashIndex;
   memset(characterEncountered, 0, sizeof(characterEncountered));
   for (hashIndex = 0; hashIndex < HASHNUM; hashIndex++) {
-    ContractionTableOffset bucket = tableHeader->buckets[hashIndex];
+    ContractionTableOffset bucket = tableHeader->rules[hashIndex];
     while (bucket) {
-      const ContractionTableEntry *entry = CTE(tableHeader, bucket);
-      switch (entry->opcode) {
+      const ContractionTableRule *rule = CTR(tableHeader, bucket);
+      switch (rule->opcode) {
         default:
-          if (!entry->replen) {
-            if (!auditCharacters(characterEncountered, &entry->findrep[0], entry->findlen)) {
+          if (!rule->replen) {
+            if (!auditCharacters(characterEncountered, &rule->findrep[0], rule->findlen)) {
               ok = 0;
             }
           }
           break;
         case CTO_Replace:
-          if (!auditCharacters(characterEncountered, &entry->findrep[entry->findlen], entry->replen)) {
+          if (!auditCharacters(characterEncountered, &rule->findrep[rule->findlen], rule->replen)) {
             ok = 0;
           }
           break;
         case CTO_Literal:
           break;
       }
-      bucket = entry->next;
+      bucket = rule->next;
     }
   }
   return ok;
@@ -744,6 +884,7 @@ auditTable (void) {
 
 void *
 compileContractionTable (const char *fileName) { /*compile source table into a table in memory */
+  int ok = 0;
   ContractionTableOffset headerOffset;
   errorCount = 0;
 
@@ -751,28 +892,56 @@ compileContractionTable (const char *fileName) { /*compile source table into a t
   tableSize = 0;
   tableUsed = 0;
 
-  opcodesTable = NULL;
-  opcodesSize = 0;
-  opcodesUsed = 0;
+  if (!opcodeLengths[0]) {
+    ContractionTableOpcode opcode;
+    for (opcode=0; opcode<CTO_None; ++opcode)
+      opcodeLengths[opcode] = strlen(opcodeNames[opcode]);
+  }
+
+  {
+    const char *locale = getLocale();
+    originalLocale = locale? strdup(locale): NULL;
+  }
+  setLocale("C");
+  noLocale = 0;
 
   if (allocateBytes(NULL, &headerOffset, sizeof(*tableHeader), __alignof__(*tableHeader))) {
     if (headerOffset == 0) {
-      cacheCharacterAttributes();
-      if (processFile(fileName))
-        auditTable();
-      if (errorCount)
-        LogPrint(LOG_ERR, "%d %s in contraction table '%s'.",
-                 errorCount, ((errorCount == 1)? "error": "errors"), fileName);
+      if (allocateCharacterClasses()) {
+        if (processFile(fileName)) {
+          if (auditTable()) {
+            ok = 1;
+            if (!noLocale) {
+              if (saveString(NULL, &tableHeader->locale, getLocale())) {
+                cacheCharacterAttributes();
+              } else {
+                ok = 0;
+              }
+            }
+          }
+        }
+        deallocateCharacterClasses();
+      }
     } else {
       compileError(NULL, "contraction table header not allocated at offset 0.");
     }
   }
 
-  if (opcodesTable) {
-    free(opcodesTable);
-    opcodesTable = NULL;
+  if (originalLocale) {
+    setLocale(originalLocale);
+    free(originalLocale);
+    originalLocale = NULL;
   }
-  return (void *) tableHeader;
+
+  if (!ok) {
+    free(tableHeader);
+    tableHeader = NULL;
+  }
+
+  if (errorCount)
+    LogPrint(LOG_WARNING, "%d %s in contraction table '%s'.",
+             errorCount, ((errorCount == 1)? "error": "errors"), fileName);
+  return (void *)tableHeader;
 }
 
 int
@@ -781,3 +950,8 @@ destroyContractionTable (void *contractionTable) {
   free(table);
   return 1;
 }
+
+void
+fixContractionTablePath (char **path) {
+  fixPath(path, CONTRACTION_TABLE_EXTENSION, CONTRACTION_TABLE_PREFIX);
+}
