@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -26,54 +26,114 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 
 #include "misc.h"
 #include "system.h"
+#include "drivers.h"
 #include "scr.h"
 #include "scr_frozen.h"
 #include "scr_help.h"
 #include "scr_real.h"
+#include "scr.auto.h"
 
-static BaseScreen *currentScreen;
 static HelpScreen helpScreen;
 static FrozenScreen frozenScreen;                
 static MainScreen mainScreen;
-static const ScreenDriver *screenDriver = NULL;
-static void *screenObject;
+static BaseScreen *currentScreen = &mainScreen.base;
+
+#define SCRSYMBOL noScreen
+#define DRIVER_NAME NoScreen
+#define DRIVER_CODE no
+#define DRIVER_COMMENT "no screen support"
+#define DRIVER_VERSION ""
+#define DRIVER_DEVELOPERS ""
+#include "scr_driver.h"
+
+static void
+scr_initialize (MainScreen *main) {
+  initializeMainScreen(main);
+}
+
+const ScreenDriver *screen = &noScreen;
+
+const char *const *
+getScreenParameters (const ScreenDriver *driver) {
+  return driver->parameters;
+}
+
+const DriverDefinition *
+getScreenDriverDefinition (const ScreenDriver *driver) {
+  return &driver->definition;
+}
+
+int
+haveScreenDriver (const char *code) {
+  return haveDriver(code, SCREEN_DRIVER_CODES, driverTable);
+}
+
+const char *
+getDefaultScreenDriver (void) {
+  return getDefaultDriver(driverTable);
+}
+
+const ScreenDriver *
+loadScreenDriver (const char *code, void **driverObject, const char *driverDirectory) {
+  return loadDriver(code, driverObject,
+                    driverDirectory, driverTable,
+                    "screen", 'x', "scr",
+                    &noScreen, &noScreen.definition);
+}
 
 void
-initializeAllScreens (const char *identifier, const char *driverDirectory) {
-  initializeHelpScreen(&helpScreen);
-  initializeFrozenScreen(&frozenScreen);
+initializeScreen (void) {
+  screen->initialize(&mainScreen);
+}
 
-  {
-    if (!(screenDriver = loadScreenDriver(identifier, &screenObject, driverDirectory))) {
-      screenDriver = &noScreen;
-      screenObject = NULL;
+int
+constructScreenDriver (char **parameters) {
+  initializeScreen();
+  if (mainScreen.processParameters(parameters)) {
+    if (mainScreen.construct()) {
+      return 1;
+    } else {
+      LogPrint(LOG_DEBUG, "screen driver initialization failed: %s",
+               screen->definition.code);
     }
+  }
 
-    identifyScreenDriver(screenDriver);
-    screenDriver->initialize(&mainScreen);
+  return 0;
+}
+
+void
+destructScreenDriver (void) {
+  mainScreen.destruct();
+}
+
+void
+identifyScreenDriver (const ScreenDriver *driver, int full) {
+  identifyDriver("Screen", &driver->definition, full);
+}
+
+void
+identifyScreenDrivers (int full) {
+  const DriverEntry *entry = driverTable;
+  while (entry->address) {
+    const ScreenDriver *driver = entry++->address;
+    identifyScreenDriver(driver, full);
   }
 }
 
 void
-closeAllScreens (void) {
-  frozenScreen.close();
-  helpScreen.close();
-  mainScreen.close();
+constructSpecialScreens (void) {
+  initializeHelpScreen(&helpScreen);
+  initializeFrozenScreen(&frozenScreen);
+}
 
-  if (screenDriver) {
-    screenDriver = NULL;
-
-    if (screenObject) {
-      unloadSharedObject(screenObject);
-      screenObject = NULL;
-    }
-  }
+void
+destructSpecialScreens (void) {
+  frozenScreen.destruct();
+  helpScreen.destruct();
 }
 
 
@@ -95,10 +155,12 @@ selectScreen (void) {
     {0         , &mainScreen.base}
   };
   const ScreenEntry *entry = screenEntries;
+
   while (entry->which) {
     if (entry->which & activeScreens) break;
     ++entry;
   }
+
   currentScreen = entry->screen;
 }
 
@@ -132,24 +194,40 @@ validateScreenBox (const ScreenBox *box, int columns, int rows) {
   return 0;
 }
 
+void
+setScreenMessage (const ScreenBox *box, unsigned char *buffer, ScreenCharacterProperty property, const char *message) {
+  int count = box->width * box->height;
+
+  if (property == SCR_TEXT) {
+    int length = strlen(message) - box->left;
+    if (length < 0) length = 0;
+    if (length > box->width) length = box->width;
+
+    memset(buffer, ' ', count);
+    if (length) memcpy(buffer, message+box->left, length);
+  } else {
+    memset(buffer, 0X07, count);
+  }
+}
+
 void
 describeScreen (ScreenDescription *description) {
-  currentScreen->describe(description);
+  describeBaseScreen(currentScreen, description);
 }
 
 int
-readScreen (short left, short top, short width, short height, unsigned char *buffer, ScreenMode mode) {
+readScreen (short left, short top, short width, short height, unsigned char *buffer, ScreenCharacterProperty property) {
   ScreenBox box;
   box.left = left;
   box.top = top;
   box.width = width;
   box.height = height;
-  return currentScreen->read(box, buffer, mode);
+  return currentScreen->read(box, buffer, property);
 }
 
 int
 insertKey (ScreenKey key) {
-  return currentScreen->insert(key);
+  return currentScreen->insertKey(key);
 }
 
 int
@@ -167,92 +245,67 @@ insertString (const char *string) {
 
 int
 routeCursor (int column, int row, int screen) {
-  return currentScreen->route(column, row, screen);
+  return currentScreen->routeCursor(column, row, screen);
 }
 
 int
-setPointer (int column, int row) {
-  return currentScreen->point(column, row);
+highlightScreenRegion (int left, int right, int top, int bottom) {
+  return currentScreen->highlightRegion(left, right, top, bottom);
 }
 
 int
-getPointer (int *column, int *row) {
-  return currentScreen->pointer(column, row);
+unhighlightScreenRegion (void) {
+  return currentScreen->unhighlightRegion();
+}
+
+int
+getScreenPointer (int *column, int *row) {
+  return currentScreen->getPointer(column, row);
 }
 
 int
 selectVirtualTerminal (int vt) {
-  return currentScreen->selectvt(vt);
+  return currentScreen->selectVirtualTerminal(vt);
 }
 
 int
 switchVirtualTerminal (int vt) {
-  return currentScreen->switchvt(vt);
+  return currentScreen->switchVirtualTerminal(vt);
 }
 
 int
 currentVirtualTerminal (void) {
-  return currentScreen->currentvt();
+  return currentScreen->currentVirtualTerminal();
 }
 
 int
 userVirtualTerminal (int number) {
-  return mainScreen.uservt(number);
+  return mainScreen.userVirtualTerminal(number);
 }
 
 int
-executeScreenCommand (int cmd) {
-  return currentScreen->execute(cmd);
-}
-
-
-const char *const *
-getScreenParameters (void) {
-  return screenDriver->parameters;
-}
-
-const char *
-getScreenDriverCode (void) {
-  return screenDriver->code;
-}
-
-int
-openMainScreen (char **parameters) {
-  if (mainScreen.prepare(parameters)) {
-    if (mainScreen.open()) {
-      if (mainScreen.setup()) {
-        currentScreen = &mainScreen.base;
-        return 1;
-      }
-      mainScreen.close();
-    }
-  }
-  return 0;
+executeScreenCommand (int command) {
+  return currentScreen->executeCommand(command);
 }
 
 
 int
-openRoutingScreen (void) {
+constructRoutingScreen (void) {
   /* This function should be used in a forked process. Though we want to
    * have a separate file descriptor for the main screen from the one used
    * in the main thread.  So we close and reopen the device.
    */
-  mainScreen.close();
-  return mainScreen.open();
+  mainScreen.destruct();
+  return mainScreen.construct();
 }
 
 void
-describeRoutingScreen (ScreenDescription *desscription) {
-  mainScreen.base.describe(desscription);
-}
-
-void
-closeRoutingScreen (void) {
-  mainScreen.close();
+destructRoutingScreen (void) {
+  mainScreen.destruct();
 }
 
 
-static int helpOpened = 0;
+static int helpConstructed = 0;
 
 int
 isHelpScreen (void) {
@@ -266,7 +319,7 @@ haveHelpScreen (void) {
 
 int
 activateHelpScreen (void) {
-  if (!helpOpened) return 0;
+  if (!helpConstructed) return 0;
   activateScreen(SCR_HELP);
   return 1;
 }
@@ -277,15 +330,15 @@ deactivateHelpScreen (void) {
 }
 
 int
-openHelpScreen (const char *file) {
-  return helpOpened = helpScreen.open(file);
+constructHelpScreen (const char *file) {
+  return helpConstructed = helpScreen.construct(file);
 }
 
 void
-closeHelpScreen (void) {
-  if (helpOpened) {
-    helpScreen.close();
-    helpOpened = 0;
+destructHelpScreen (void) {
+  if (helpConstructed) {
+    helpScreen.destruct();
+    helpConstructed = 0;
   }
 }
 
@@ -317,7 +370,7 @@ haveFrozenScreen (void) {
 
 int
 activateFrozenScreen (void) {
-  if (haveFrozenScreen() || !frozenScreen.open(&mainScreen.base)) return 0;
+  if (haveFrozenScreen() || !frozenScreen.construct(&mainScreen.base)) return 0;
   activateScreen(SCR_FROZEN);
   return 1;
 }
@@ -325,7 +378,7 @@ activateFrozenScreen (void) {
 void
 deactivateFrozenScreen (void) {
   if (haveFrozenScreen()) {
-    frozenScreen.close();
+    frozenScreen.destruct();
     deactivateScreen(SCR_FROZEN);
   }
 }
