@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _SCR_REAL_H
-#define _SCR_REAL_H
+#ifndef BRLTTY_INCLUDED_SCR_REAL
+#define BRLTTY_INCLUDED_SCR_REAL
 
 #ifdef __cplusplus
 extern "C" {
@@ -30,4 +30,4 @@ extern void initializeRealScreen (MainScreen *);
 }
 #endif /* __cplusplus */
 
-#endif /* _SCR_REAL_H */
+#endif /* BRLTTY_INCLUDED_SCR_REAL */
