@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,28 +15,35 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _TBL_H
-#define _TBL_H
+#ifndef BRLTTY_INCLUDED_TBL
+#define BRLTTY_INCLUDED_TBL
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-typedef void (*TranslationTableReporter) (const char *message);
+#include <stdio.h>
+
+#include "brl.h"
 
 #define TBL_UNDEFINED 0X1
 #define TBL_DUPLICATE 0X2
+#define TBL_UNUSED    0X4
 
 extern int loadTranslationTable (
-  const char *file,
-  TranslationTable *table,
-  TranslationTableReporter report,
+  const char *path,
+  FILE *file,
+  TranslationTable table,
   int options
 );
-extern void reverseTranslationTable (TranslationTable *from, TranslationTable *to);
+
+extern void reverseTranslationTable (TranslationTable from, TranslationTable to);
+
+extern void fixTextTablePath (char **path);
+extern void fixAttributesTablePath (char **path);
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* _TBL_H */
+#endif /* BRLTTY_INCLUDED_TBL */
