@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,16 +15,16 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-typedef struct {
+struct UsbDeviceExtensionStruct {
   char *path;
   int file;
   int timeout;
-} UsbDeviceExtension;
+};
 
-typedef struct {
+struct UsbEndpointExtensionStruct {
   int file;
   int timeout;
-} UsbEndpointExtension;
+};
 
 static int
 usbSetTimeout (int file, int new, int *old) {
@@ -53,6 +53,13 @@ usbResetDevice (UsbDevice *device) {
   return 0;
 }
 
+int
+usbDisableAutosuspend (UsbDevice *device) {
+  errno = ENOSYS;
+  LogError("USB device autosuspend disable");
+  return 0;
+}
+
 int
 usbSetConfiguration (
   UsbDevice *device,
@@ -291,9 +298,7 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
 }
 
 void
-usbDeallocateEndpointExtension (UsbEndpoint *endpoint) {
-  UsbEndpointExtension *eptx = endpoint->extension;
-
+usbDeallocateEndpointExtension (UsbEndpointExtension *eptx) {
   if (eptx->file != -1) {
     close(eptx->file);
     eptx->file = -1;
@@ -303,9 +308,7 @@ usbDeallocateEndpointExtension (UsbEndpoint *endpoint) {
 }
 
 void
-usbDeallocateDeviceExtension (UsbDevice *device) {
-  UsbDeviceExtension *devx = device->extension;
-
+usbDeallocateDeviceExtension (UsbDeviceExtension *devx) {
   if (devx->file != -1) {
     close(devx->file);
     devx->file = -1;
