@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2007 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,29 +17,23 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 
-#include "Programs/misc.h"
+#include "misc.h"
 
 //#define SPK_HAVE_EXPRESS
 //#define SPK_HAVE_TRACK
 //#define SPK_HAVE_RATE
 //#define SPK_HAVE_VOLUME
-#include "Programs/spk_driver.h"
-
-static void
-spk_identify (void) {
-}
+#include "spk_driver.h"
 
 static int
-spk_open (char **parameters) {
+spk_construct (char **parameters) {
   return 0;
 }
 
 static void
-spk_close (void) {
+spk_destruct (void) {
 }
 
 static void
